// pvect.h   1996.9.17 programmed by Masaaki Wada
// 1997.6.19 complex operations added,
//				and the operator for the inner product changed
#ifndef	_pvect_
#define _pvect_

#include	<math.h>

// pvect ----- plane vector
class pvect {
public:
	double		x, y;
	
	pvect( double _x = 0.0, double _y = 0.0 )
		: x( _x ), y( _y ) {}
	
	void		set( double _x, double _y )
		{ x = _x; y = _y; }
	
	pvect		operator +=( pvect v )
		{ x += v.x; y += v.y; return ( *this ); }
	pvect		operator -=( pvect v )
		{ x -= v.x; y -= v.y; return ( *this ); }

	pvect		operator *=( double c )
		{ x *= c; y *= c; return ( *this ); }
	pvect		operator /=( double c )
		{ x /= c; y /= c; return ( *this ); }

	pvect		operator *=( pvect v )			// complex multiplication
		{
			double	xx = x * v.x - y * v.y;
			y = y * v.x + x * v.y;
			x = xx;
			return ( *this );
		}
	pvect		operator /=( pvect v )			// complex division
		{
			double	nrm = v.norm();
			if ( nrm > 0.0 ) {
				double	xx = ( x * v.x + y * v.y ) / nrm;
				y = ( y * v.x - x * v.y ) / nrm;
				x = xx;
				}
			return ( *this );
		}

	double		norm( void )
		{ return ( x * x + y * y ); }
	double		abs( void )
		{ return sqrt( norm() ); }

	pvect		conjugated( void )				// complex conjugate
		{ return pvect( x, -y ); }
	pvect		conjugate( void )
		{ y = -y; return ( *this ); }

	pvect		normalized( void ) {
		double	a;
		if ( ( a = abs() ) > 0.0 )
			return pvect( x / a, y / a );
		else
			return ( *this );
		}
	pvect		normalize( void ) {
		double	a;
		if ( ( a = abs() ) > 0.0 )
			{ x /= a; y /= a; }
		return ( *this );
		}
	pvect		rotated( double t ) {
		double	cos_t = cos( t ), sin_t = sin( t );
		return pvect( cos_t * x - sin_t * y, sin_t * x + cos_t * y );
		}
	pvect		rotate( double t ) {
		double	_x = x;
		double	cos_t = cos( t ), sin_t = sin( t );
		x = cos_t * _x - sin_t * y;
		y = sin_t * _x + cos_t * y;
		return ( *this );
		}
};

// unary operators
inline pvect	operator +( pvect v )
	{ return v; }
inline pvect	operator -( pvect v )
	{ return pvect( -v.x, -v.y ); }
inline pvect	operator ~( pvect v )			// rotated left pi/2
	{ return pvect( -v.y, v.x ); }

// binary operators
inline pvect	operator +( pvect u, pvect v )
	{ return ( u += v ); }
inline pvect	operator -( pvect u, pvect v )
	{ return ( u -= v ); }
inline pvect	operator *( double c, pvect v )
	{ return ( v *= c ); }
inline pvect	operator *( pvect v, double c )
	{ return ( v *= c ); }
inline pvect	operator /( pvect v, double c )
	{ return ( v /= c ); }
inline pvect	operator *( pvect u, pvect v )	// complex product
	{ return ( u *= v ); }
inline pvect	operator /( pvect u, pvect v )	// complex quotient
	{ return ( u /= v ); }
inline double	operator |( pvect u, pvect v )	// inner product
	{ return ( u.x * v.x + u.y * v.y ); }
inline double	operator ^( pvect u, pvect v )	// outer product
	{ return ( u.x * v.y - u.y * v.x ); }

// logical operators
inline int		operator ==( pvect u, pvect v )
	{ return ( u.x == v.x && u.y == v.y ); }
inline int		operator !=( pvect u, pvect v )
	{ return ( ! ( u == v ) ); }

// other
inline double	norm( pvect v )
	{ return v.norm(); }
inline double	abs( pvect v )
	{ return v.abs(); }
inline pvect	conj( pvect v )
	{ return v.conjugated(); }
inline double	det( pvect u, pvect v )
	{ return ( u ^ v ); }


#endif

